@@ -39,10 +39,20 @@ atspi_component_interface_init (AtspiComponent *component)
 {
 }
 
+static void
+atspi_document_interface_init (AtspiDocument *document)
+{
+}
+
 atspi_editable_text_interface_init (AtspiEditableText *editable_text)
 {
 }
 
+static void
+atspi_hypertext_interface_init (AtspiHypertext *hypertext)
+{
+}
+
 static void
 atspi_image_interface_init (AtspiImage *image)
 {
@@ -72,7 +82,9 @@ G_DEFINE_TYPE_WITH_CODE (AtspiAccessible, atspi_accessible, ATSPI_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (ATSPI_TYPE_ACTION, atspi_action_interface_init)
                          G_IMPLEMENT_INTERFACE (ATSPI_TYPE_COLLECTION, atspi_collection_interface_init)
                          G_IMPLEMENT_INTERFACE (ATSPI_TYPE_COMPONENT, atspi_component_interface_init)
+                         G_IMPLEMENT_INTERFACE (ATSPI_TYPE_DOCUMENT, atspi_document_interface_init)
                          G_IMPLEMENT_INTERFACE (ATSPI_TYPE_EDITABLE_TEXT, atspi_editable_text_interface_init)
+                         G_IMPLEMENT_INTERFACE (ATSPI_TYPE_HYPERTEXT, atspi_hypertext_interface_init)
                          G_IMPLEMENT_INTERFACE (ATSPI_TYPE_IMAGE, atspi_image_interface_init)
                          G_IMPLEMENT_INTERFACE (ATSPI_TYPE_SELECTION, atspi_selection_interface_init)
                          G_IMPLEMENT_INTERFACE (ATSPI_TYPE_TABLE, atspi_table_interface_init)
@@ -230,12 +242,12 @@ atspi_role_get_name (AtspiRole role)
 gchar *
 atspi_accessible_get_name (AtspiAccessible *obj, GError **error)
 {
-  g_return_val_if_fail (obj != NULL, NULL);
+  g_return_val_if_fail (obj != NULL, g_strdup (""));
   if (!(obj->cached_properties & ATSPI_CACHE_NAME))
   {
     if (!_atspi_dbus_get_property (obj, atspi_interface_accessible, "Name", error,
                                    "s", &obj->name))
-      return NULL;
+      return g_strdup ("");
     obj->cached_properties |= ATSPI_CACHE_NAME;
   }
   return g_strdup (obj->name);
@@ -253,12 +265,12 @@ atspi_accessible_get_name (AtspiAccessible *obj, GError **error)
 gchar *
 atspi_accessible_get_description (AtspiAccessible *obj, GError **error)
 {
-  g_return_val_if_fail (obj != NULL, NULL);
+  g_return_val_if_fail (obj != NULL, g_strdup (""));
 
   if (!(obj->cached_properties & ATSPI_CACHE_DESCRIPTION))
   {
     if (!_atspi_dbus_call (obj, atspi_interface_accessible, "GetDescription", NULL, "=>s", &obj->description))
-      return NULL;
+      return g_strdup ("");
     obj->cached_properties |= ATSPI_CACHE_DESCRIPTION;
   }
   return g_strdup (obj->description);
@@ -320,11 +332,19 @@ atspi_accessible_get_parent (AtspiAccessible *obj, GError **error)
  *
  **/
 gint
-atspi_accessible_get_child_count (AtspiAccessible *obj, GError *error)
+atspi_accessible_get_child_count (AtspiAccessible *obj, GError **error)
 {
   g_return_val_if_fail (obj != NULL, -1);
 
-  /* TODO: MANAGES_DESCENDANTS */
+  if (!(obj->cached_properties & ATSPI_CACHE_CHILDREN))
+  {
+    dbus_int32_t ret;
+    if (!_atspi_dbus_get_property (obj, atspi_interface_accessible,
+                                   "ChildCount", error, "i", &ret))
+      return -1;
+    return ret;
+  }
+
   return g_list_length (obj->children);
 }
 
@@ -347,7 +367,15 @@ atspi_accessible_get_child_at_index (AtspiAccessible *obj,
 
   g_return_val_if_fail (obj != NULL, NULL);
 
-  /* TODO: ManagesDescendants */
+  if (!(obj->cached_properties & ATSPI_CACHE_CHILDREN))
+  {
+    DBusMessage *reply;
+    reply = _atspi_dbus_call_partial (obj, atspi_interface_accessible,
+                                     "GetChildAtIndex", error, "i",
+                                     child_index);
+    return _atspi_dbus_return_accessible_from_message (reply);
+  }
+
   child = g_list_nth_data (obj->children, child_index);
   if (!child)
     return NULL;
@@ -368,10 +396,18 @@ gint
 atspi_accessible_get_index_in_parent (AtspiAccessible *obj, GError **error)
 {
   GList *l;
-  gint i;
+  gint i = 0;
 
   g_return_val_if_fail (obj != NULL, -1);
   if (!obj->accessible_parent) return -1;
+  if (!(obj->accessible_parent->cached_properties & ATSPI_CACHE_CHILDREN))
+  {
+    dbus_uint32_t ret = -1;
+    _atspi_dbus_call (obj, atspi_interface_accessible,
+                      "GetIndexInParent", NULL, "=>u", &ret);
+    return ret;
+  }
+
   l = obj->accessible_parent->children;
   while (l)
   {
@@ -470,10 +506,13 @@ atspi_accessible_get_role_name (AtspiAccessible *obj, GError **error)
 {
   char *retval = NULL;
 
-  g_return_val_if_fail (obj != NULL, g_strdup ("invalid"));
+  g_return_val_if_fail (obj != NULL, NULL);
 
   _atspi_dbus_call (obj, atspi_interface_accessible, "GetRoleName", error, "=>s", &retval);
 
+  if (!retval)
+    retval = g_strdup ("");
+
   return retval;
 }
 
@@ -493,10 +532,13 @@ atspi_accessible_get_localized_role_name (AtspiAccessible *obj, GError **error)
 {
   char *retval = NULL;
 
-  g_return_val_if_fail (obj != NULL, g_strdup ("invalid"));
+  g_return_val_if_fail (obj != NULL, NULL);
 
   _atspi_dbus_call (obj, atspi_interface_accessible, "GetLocalizedRoleName", error, "=>s", &retval);
 
+  if (!retval)
+    return g_strdup ("");
+
   return retval;
 }
 
@@ -512,6 +554,18 @@ atspi_accessible_get_localized_role_name (AtspiAccessible *obj, GError **error)
 AtspiStateSet *
 atspi_accessible_get_state_set (AtspiAccessible *obj)
 {
+  if (!(obj->cached_properties & ATSPI_CACHE_STATES))
+  {
+    DBusMessage *reply;
+    DBusMessageIter iter;
+    reply = _atspi_dbus_call_partial (obj, atspi_interface_accessible,
+                                      "GetState", NULL, "");
+    _ATSPI_DBUS_CHECK_SIG (reply, "au", NULL);
+    dbus_message_iter_init (reply, &iter);
+    _atspi_dbus_set_state (obj, &iter);
+    dbus_message_unref (reply);
+  }
+
   return g_object_ref (obj->states);
 }
 
@@ -531,14 +585,11 @@ GHashTable *
 atspi_accessible_get_attributes (AtspiAccessible *obj, GError **error)
 {
   DBusMessage *message;
-  GHashTable *ret;
 
     g_return_val_if_fail (obj != NULL, NULL);
 
   message = _atspi_dbus_call_partial (obj, atspi_interface_accessible, "GetAttributes", error, "");
-  ret = _atspi_dbus_hash_from_message (message);
-  dbus_message_unref (message);
-  return ret;
+  return _atspi_dbus_return_hash_from_message (message);
 }
 
 /**
@@ -557,15 +608,11 @@ GArray *
 atspi_accessible_get_attributes_as_array (AtspiAccessible *obj, GError **error)
 {
   DBusMessage *message;
-  GArray *ret;
 
     g_return_val_if_fail (obj != NULL, NULL);
 
   message = _atspi_dbus_call_partial (obj, atspi_interface_accessible, "GetAttributes", error, "");
-  ret = _atspi_dbus_attribute_array_from_message (message);
-  if (message)
-    dbus_message_unref (message);
-  return ret;
+  return _atspi_dbus_return_attribute_array_from_message (message);
 }
 
 /**
@@ -592,7 +639,7 @@ atspi_accessible_get_application (AtspiAccessible *obj, GError **error)
   }
 }
 
-/* Applicatio-specific methods */
+/* Application-specific methods */
 
 /**
  * atspi_accessible_get_toolkit_name:
@@ -936,6 +983,23 @@ atspi_accessible_get_editable_text (AtspiAccessible *accessible)
           g_object_ref (ATSPI_EDITABLE_TEXT (accessible)) : NULL);  
 }
 
+/**
+ * atspi_accessible_get_hyperlink:
+ * @accessible: a pointer to the #AtspiAccessible object on which to operate.
+ *
+ * Get the #AtspiHyperlink associated with the given #AtspiAccessible, if
+ * supported.
+ *
+ * Returns: (transfer full): the #AtspiHyperlink object associated with
+ *          the given #AtspiAccessible, or NULL if not supported.
+ **/
+AtspiHyperlink *
+atspi_accessible_get_hyperlink (AtspiAccessible *accessible)
+{
+  return (_atspi_accessible_is_a (accessible, atspi_interface_hyperlink) ?
+          atspi_hyperlink_new (accessible->parent.app, accessible->parent.path) : NULL);
+}
+
 /**
  * atspi_accessible_get_hypertext:
  * @obj: a pointer to the #AtspiAccessible instance to query.
@@ -1025,7 +1089,7 @@ atspi_accessible_get_table (AtspiAccessible *obj)
  * Get the #AtspiTable interface for an #AtspiAccessible.
  *
  * Returns: (transfer full): a pointer to an #AtspiText interface instance, or
- *          NULL if @obj does not implement #AtspiTable.
+ *          NULL if @obj does not implement #AtspiText.
  **/
 AtspiText *
 atspi_accessible_get_text (AtspiAccessible *obj)
@@ -1373,11 +1437,77 @@ _atspi_accessible_is_a (AtspiAccessible *accessible,
       return FALSE;
     }
 
+  if (!(accessible->cached_properties & ATSPI_CACHE_INTERFACES))
+  {
+    DBusMessage *reply;
+    DBusMessageIter iter;
+    reply = _atspi_dbus_call_partial (accessible, atspi_interface_accessible,
+                                      "GetInterfaces", NULL, "");
+    _ATSPI_DBUS_CHECK_SIG (reply, "as", FALSE);
+    dbus_message_iter_init (reply, &iter);
+    _atspi_dbus_set_interfaces (accessible, &iter);
+    dbus_message_unref (reply);
+  }
+
   n = _atspi_get_iface_num (interface_name);
   if (n == -1) return FALSE;
   return (gboolean) ((accessible->interfaces & (1 << n))? TRUE: FALSE);
 }
 
+static void
+append_const_val (GArray *array, const gchar *val)
+{
+  gchar *dup = g_strdup (val);
+
+  if (dup)
+    g_array_append_val (array, dup);
+}
+
+/**
+ * atspi_accessible_get_interfaces:
+ *
+ * #obj: The #AtspiAccessible to query.
+ *
+ * Returns: (element-type gchar*) (transfer full): A #GArray of strings
+ *          describing the interfaces supported by the object.  Interfaces are
+ *          denoted in short-hand (ie, "Component", "Text", etc.)
+ **/
+GArray *
+atspi_accessible_get_interfaces (AtspiAccessible *obj)
+{
+  GArray *ret = g_array_new (TRUE, TRUE, sizeof (gchar *));
+
+  if (!ret)
+    return NULL;
+
+  g_return_val_if_fail (obj != NULL, NULL);
+
+  if (atspi_accessible_is_action (obj))
+    append_const_val (ret, "Action");
+  if (atspi_accessible_is_collection (obj))
+    append_const_val (ret, "Collection");
+  if (atspi_accessible_is_component (obj))
+    append_const_val (ret, "Component");
+  if (atspi_accessible_is_document (obj))
+    append_const_val (ret, "Document");
+  if (atspi_accessible_is_editable_text (obj))
+    append_const_val (ret, "EditableText");
+  if (atspi_accessible_is_hypertext (obj))
+    append_const_val (ret, "Hypertext");
+  if (atspi_accessible_is_image (obj))
+    append_const_val (ret, "Image");
+  if (atspi_accessible_is_selection (obj))
+    append_const_val (ret, "Selection");
+  if (atspi_accessible_is_table (obj))
+    append_const_val (ret, "Table");
+  if (atspi_accessible_is_text (obj))
+    append_const_val (ret, "Text");
+  if (atspi_accessible_is_value (obj))
+    append_const_val (ret, "Value");
+
+  return ret;
+}
+
 /* TODO: Move to a finalizer */
 static void
 cspi_object_destroyed (AtspiAccessible *accessible)
