@@ -176,6 +176,8 @@ cache_process_children_changed (AtspiEvent *event)
 
   if (!strncmp (event->type, "object:children-changed:add", 27))
   {
+    if (g_list_find (event->source->children, child))
+      return;
     GList *new_list = g_list_insert (event->source->children, g_object_ref (child), event->detail1);
     if (new_list)
       event->source->children = new_list;
@@ -239,7 +241,9 @@ cache_process_property_change (AtspiEvent *event)
 static void
 cache_process_state_changed (AtspiEvent *event)
 {
-  atspi_state_set_set_by_name (event->source->states, event->type + 21, event->detail1);
+  if (event->source->states)
+    atspi_state_set_set_by_name (event->source->states, event->type + 21,
+                                 event->detail1);
 }
 
 static dbus_bool_t
@@ -680,8 +684,8 @@ atspi_event_copy (AtspiEvent *src)
   dst->source = g_object_ref (src->source);
   dst->detail1 = src->detail1;
   dst->detail2 = src->detail2;
-  dst->any_data.g_type = src->any_data.g_type;
-  g_value_copy (&dst->any_data, &src->any_data);
+  g_value_init (&dst->any_data, G_VALUE_TYPE (&src->any_data));
+  g_value_copy (&src->any_data, &dst->any_data);
   return dst;
 }
 
@@ -824,15 +828,15 @@ atspi_dbus_handle_event (DBusConnection *bus, DBusMessage *message, void *data)
   }
   _atspi_send_event (&e);
 
-  if (!strcmp (e.type, "children-changed"))
+  if (!strncmp (e.type, "object:children-changed", 23))
   {
     cache_process_children_changed (&e);
   }
-  else if (!strcmp (e.type, "property-change"))
+  else if (!strncmp (e.type, "object:property-change", 22))
   {
     cache_process_property_change (&e);
   }
-  else if (!strcmp (e.type, "state-changed"))
+  else if (!strncmp (e.type, "object:state-changed", 20))
   {
     cache_process_state_changed (&e);
   }
