@@ -590,8 +590,15 @@ impl_get_ChildCount (DBusMessageIter * iter, void *user_data)
 {
   SpiRegistry *reg = SPI_REGISTRY (user_data);
   dbus_int32_t rv = reg->apps->len;
+  dbus_bool_t result;
+  DBusMessageIter iter_variant;
 
-  return dbus_message_iter_append_basic (iter, DBUS_TYPE_INT32, &rv);
+  if (!dbus_message_iter_open_container (iter, DBUS_TYPE_VARIANT, "i",
+                                         &iter_variant))
+    return FALSE;
+  result = dbus_message_iter_append_basic (&iter_variant, DBUS_TYPE_INT32, &rv);
+  dbus_message_iter_close_container (iter, &iter_variant);
+  return result;
 }
 
 static DBusMessage *
@@ -679,7 +686,7 @@ static DBusMessage *
 impl_GetRole (DBusConnection * bus, DBusMessage * message, void * user_data)
 {
   DBusMessage *reply;
-  dbus_uint32_t rv = 0;
+  dbus_uint32_t rv = 14;	/* TODO: Get DESKTOP_FRAME from somewhere */
 
   reply = dbus_message_new_method_return (message);
   dbus_message_append_args (reply, DBUS_TYPE_UINT32, &rv, DBUS_TYPE_INVALID);
@@ -691,7 +698,7 @@ impl_GetRoleName (DBusConnection * bus,
                   DBusMessage * message, void *user_data)
 {
   DBusMessage *reply;
-  const char *role_name = "unknown";
+  const char *role_name = "desktop frame";
 
   reply = dbus_message_new_method_return (message);
   dbus_message_append_args (reply, DBUS_TYPE_STRING, &role_name,
@@ -705,7 +712,7 @@ impl_GetLocalizedRoleName (DBusConnection * bus,
 {
   /* TODO - Localize this */
   DBusMessage *reply;
-  const char *role_name = "unknown";
+  const char *role_name = "desktop frame";
 
   reply = dbus_message_new_method_return (message);
   dbus_message_append_args (reply, DBUS_TYPE_STRING, &role_name,
